package	jp.co.sds_corp.transfer.output;

import java.util.Iterator;
import java.util.Vector;

import jp.co.sds_corp.transfer.IData;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.layout.ILayout;
import jp.co.sds_corp.transfer.layout.PatternLayout;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;
import jp.co.sds_corp.util.db.DbAccess;

/**
* データをDBに出力します。
XMLは以下の形式です。
<PRE>
&lt;Output name="Db" class="jp.co.sds_corp.transfer.output.DbOutput"&gt;
</PRE>
db要素で、DBの接続先を指定します。
この要素の詳細については、{@link DbAccess}を
参照してください。
<PRE>
&lt;db&gt;
	&lt;key&gt;db_key&lt;/db&gt;
&lt;/db&gt;
</PRE>
<P>
sql要素で、SQL文を指定します。sql要素は複数指定することが出来ます。
この要素の詳細については、{@link PatternLayout}を参照してください。
<PRE>
&lt;sql class="jp.co.sds_corp.transfer.layout.PatternLayout"&gt;
	update tbl set bid=%B, ask=%A where ccy1='%C' and ccy2='%c'
&lt;/sql&gt;
&lt;/Output&gt;
</PRE>
*/
public class DbOutput extends AbstractOutput
{
	private static final long serialVersionUID = 1L;
	private DbAccess	db;
	private Vector<ILayout>		vecLayout;

	/**
	* 指定された初期情報でDbOutputを初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		IConfiguration dbconf = config.getChild( "db" );
		db = new DbAccess( dbconf );

		vecLayout = new Vector<ILayout>();
		Iterator<IConfiguration> itr = config.getChilds( "sql" );
		while ( itr.hasNext()) {
			try {
				IConfiguration c = (IConfiguration )itr.next();
				ILayout layout = createLayout( c );
				vecLayout.add( layout );
			} catch ( Exception e ) {
				CmnLog.warn( this, "layout creation error", e );
			}
		}
	}

	/**
	* サービスを開始します。
	*/
	public void start() {
		try {
			db.connect();
		} catch ( Exception e ) {
			CmnLog.error( this, "connect error", e );
		}
	}

	/**
	* レイアウト情報を生成します。
	* @param	config	初期情報
	* @return	レイアウト情報
	*/
	protected ILayout createLayout( IConfiguration config ) throws Exception {
		Object iObj = ConfigUtility.createInstance( config );
		ILayout ret = (ILayout )iObj;
		ret.setFormat( config.get( "format", null ));
		return ret;
	}

	/**
	* データに何らかの変更があったときに呼び出されます。
	* @param	e		イベントソースおよびデータの変更内容を指定します。
	*/
	public void received( ReceiveEvent e ) throws Exception {
		if ( isDebug() ) {
			CmnLog.debug( this, e.toString() );
		}
		IData data = e.getData();
		try {
			for ( int i = 0; i < vecLayout.size(); i++ ) {
				ILayout layout = (ILayout )vecLayout.get( i );
				String sql = layout.format( data );
				db.execute( sql );
			}
			db.commit();
		} catch ( Exception ex ) {
			CmnLog.error( this, "sql execute error", ex );
			db.rollback();
		}
	}

	/**
	* DbAccessを返します。
	*/
	public DbAccess getDb() {
		return db;
	}

	/**
	* この出力先の文字列表現を返します。
	* @return	この出力先の文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += ":";
		for ( int i = 0; i < vecLayout.size(); i++ ) {
			ILayout layout = (ILayout )vecLayout.get( i );
			if ( i > 0 ) {
				ret += ",";
			}
			ret += "sql=[" + layout + "]";
		}
		return ret;
	}
}
