package	jp.co.sds_corp.transfer.output;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;

import jp.co.sds_corp.transfer.IData;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.layout.ILayout;
import jp.co.sds_corp.transfer.layout.PatternLayout;
import jp.co.sds_corp.transfer.layout.SimplePatternLayout;
import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;

/**
* データをファイルに出力します。
XMLは以下の形式です。
<PRE>
&lt;Output name="File" class="jp.co.sds_corp.transfer.output.FileOutput"&gt;
</PRE>
dir要素で出力先のディレクトリを指定します。
<PRE>
&lt;dir&gt;data/hist&lt;/dir&gt;
</PRE>
file要素で出力するファイル名を指定します。
dir要素とfile要素を分ける必要はありません。
file要素にPATHごと指定することが可能です。
<PRE>
&lt;file&gt;data.dat&lt;/file&gt;
</PRE>
file要素では{@link SimplePatternLayout}を使用することが可能です。
<PRE>
&lt;file&gt;%d{yyyyMMdd}.dat&lt;/file&gt;
</PRE>
fastopen要素をtrueに設定すると、データ受信するたびにファイルをオープンするかを指定します。
trueを設定すると毎回ファイルをオープンしないので、処理速度は向上しますが
一部のOSでは、他のプロセスがファイルを監視する場合に、遅延が発生する場合があります。
デフォルトはfalseです。
<PRE>
&lt;fastopen&gt;true&lt;/fastopen&gt;
</PRE>
<P>
layout要素で、データのフォーマット情報を指定します。
この要素の詳細については、{@link PatternLayout}を参照してください。
<PRE>
&lt;layout class="jp.co.sds_corp.transfer.layout.PatternLayout"&gt;
	&lt;format&gt;%C\t%c\t%T\t%B\t%A{0.00}\t%d{yyyy/MM/dd HH:mm:ss}\n&lt;/format&gt;
&lt;/layout&gt;
&lt;/Output&gt;
</PRE>
*/
public class FileOutput extends AbstractOutput
{
	private static final long serialVersionUID = 1L;
	String	dir;
	String	file;
	boolean	bFast = false;
	ILayout	layout = null;
	ILayout fileLayout;

	/**
	* 指定された初期情報でFileOutputを初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		dir = config.get( "dir", null );
		file = config.get( "file", null );
		if ( file != null ) {
			fileLayout = new SimplePatternLayout();
			fileLayout.setFormat( file );
		}
		bFast = config.getBoolean( "fastopen", false );
		try {
			IConfiguration c = config.getChild( "layout" );
			layout = createLayout( c );
		} catch ( Exception e ) {
			CmnLog.warn( this, "layout creation error", e );
		}
	}

	/**
	* レイアウト情報を生成します。
	* @param	config	初期情報
	* @return	レイアウト情報
	*/
	protected ILayout createLayout( IConfiguration config ) throws Exception {
		Object iObj = ConfigUtility.createInstance( config );
		ILayout ret = (ILayout )iObj;
		ret.setFormat( config.get( "format", null ));
		return ret;
	}

	/**
	* 高速出力フラグを返します。
	* @return	高速出力モードの場合true
	*/
	public boolean isFastOpen() {
		return bFast;
	}

	/**
	* 出力するディレクトリを返します。
	* @return	出力するディレクトリ
	*/
	public String getDir() {
		return dir;
	}
	/**
	* 出力するファイル名を返します。
	* @return	出力するファイル名
	*/
	public String getFile() {
		return file;
	}

	/**
	* ファイル名を変換するlayout(fileLayout)を設定します。
	* @param	var		設定するデータ
	*/
	public void setFileLayout( ILayout var ) {
		this.fileLayout = var ;
	}
	/**
	* ファイル名を変換するlayout(fileLayout)を取得します。
	* @return	fileLayoutの内容
	*/
	public ILayout getFileLayout() {
		return fileLayout;
	}

	/**
	* 出力するファイル名を返します。
	* @return	出力するファイル名
	*/
	public String getFileName( IData data ) {
		String  fn;
		if ( dir == null ) {
			fn = "";
		} else {
			fn = dir + File.separator;
		}
		if ( file == null ) {
			String sysdate = new CmnDate().getString();
			fn = fn + "data." + sysdate;
		} else {
			fn = fn + fileLayout.format( data );
		}
		return fn;
	}

	/**
	* 指定されたファイルのPATHを検査します。
	* 検査した親ディレクトリが存在しない場合には、親ディレクトリを作成します。
	*/
	protected void checkDir( String file ) throws Exception {
		File parent = new File( file ).getParentFile();
		if ( !parent.exists()) {
			parent.mkdirs();
		}
	}

	PrintWriter pw;
	String filename;

	/**
	* 出力先を取得します。
	* @param	toF		出力するファイル名
	*/
	public PrintWriter getWriter( String toF ) throws Exception {
		if ( bFast ) {
			if ( !toF.equals( filename )) {
				checkDir( toF );
				pw = new PrintWriter( new FileOutputStream( toF, true ));
			}
		} else {
			checkDir( toF );
			pw = new PrintWriter( new FileOutputStream( toF, true ));
		}
		return pw;
	}

	/**
	* データに何らかの変更があったときに呼び出されます。
	* @param	e		イベントソースおよびデータの変更内容を指定します。
	*/
	public void received( ReceiveEvent e ) throws Exception {
		if ( isDebug() ) {
			CmnLog.debug( this, e.toString() );
		}
		IData data = e.getData();
		String toF = getFileName( data );
		pw = getWriter( toF );
		String buf;
		if ( layout == null ) {
			buf = data.toString();
		} else {
			buf = layout.format( data );
		}
		pw.print( buf );
		pw.flush();
		filename = toF;
		if ( !bFast ) {
			pw.flush();
			pw.close();
		}
	}

	/**
	* この出力先の文字列表現を返します。
	* @return	この出力先の文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += ":dir=[" + dir + "] layout=[" + layout + "]";
		return ret;
	}
}
