package	jp.co.sds_corp.transfer.source;

import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.db.DbTable;

/**
* DBからデータを受信するクラスの基底クラスです。
* db要素で、DBの接続先を指定します。
この要素の詳細については、{@link DbAccess}を参照してください。
<PRE>
&lt;Source name="Db" class="jp.co.sds_corp.transfer.source.DbSource"&gt;
	&lt;db&gt;
		&lt;key&gt;db_key&lt;/db&gt;
	&lt;/db&gt;
	&lt;sql&gt;select * from tbl&lt;/sql&gt;
&lt;/Source&gt;
</PRE>
*/
public abstract class DbSource extends AbstractSource
{
	private static final long serialVersionUID = 1L;
	DbAccess	db;
	String	sql;
	int		interval;
	transient ServiceThread thr;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		IConfiguration dbconf = config.getChild( "db" );
		db = new DbAccess( dbconf );

		sql = config.get( "sql" );
		interval = config.getInteger( "interval", 1000 );
		thr = new ServiceThread();
		thr.setName( getName() );
	}

	/**
	* このサービスを開始します。
	*/
	public void start() {
		try {
			db.connect();
		} catch ( Exception e ) {
			CmnLog.error( this, "connect error", e );
		}
		setInterrupt( false );
		thr.start();
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		setInterrupt( true );
		thr.interrupt();
		try {
			thr.join();
		} catch ( Exception e ) {
		}
		CmnLog.debug( this, "stopped." );
	}

	/**
	* 読み込んだ行の内容を解析します。
	* @param	row		行の内容
	*/
	public abstract void parseRow( DbTable row ) throws Exception;

	protected void executeSql() throws Exception {
		DbTable tbl = new DbTable();
		db.executeQuery( sql );
		for ( ;; ) {
			if ( !db.fetch( tbl )) {
				break;
			}
			parseRow( tbl );
		}
	}

	private class ServiceThread extends Thread
	{
		public ServiceThread() {
		}

		public void run() {
			try {
				executeSql();
			} catch ( Exception e ) {
				CmnLog.error( this, "error", e );
				return;
			}
		}

	}

	/**
	* この取得元の文字列表現を返します。
	* @return	この取得元の文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += ":db=[" + db.toString() + "]";
		ret += " sql=[" + sql + "]";
		return ret;
	}

}
